      SUBROUTINE MAP_STEEL_BSMS(T10,T20,W,W1,JTLOW,JTHIGH,JT,
     &                          BS,MS,X1,XTOSL,XTOCN,XTOSL2,XTOCN2,R)
C
C  To calculate the martensite-start and bainite-start temperatures
C  Note that the bainite-start temperature is calculated on the
C  growth condition alone, i.e. it is defined by the point where
C  the driving force for diffusionless transformation exceeds the
C  stored energy of bainite
C
C H. K. D. H. Bhadeshia, University of Cambridge
C
C BS and MS are the bainite-start and martensite-start temperatures in
C   degrees centigrade
C
C JTLOW, JTHIGH and JT define the range over which calculations are done
C   JT defines the temperature interval at which calculations done
C
C T10 and T20 are calculated by subroutine MAP_STEEL_OMEGA. They are
C   functions of the substitutional solute content of the alloy.
C W and W1 represent the carbon-carbon interaction energies in
C  austenite and ferrite respectively, in Joules per mole.
C X1 is the carbon concentration in mole fraction.
C
      IMPLICIT NONE    
      DOUBLE PRECISION T10,T20,W,W1,BS,MS,X1,XTOSL,XTOCN,XTOSL2,XTOCN2,R
      DOUBLE PRECISION DCTEMP(100),FTO(100),XXTO(100),YXTO(100),VXTO(20)
      DOUBLE PRECISION WXTO(20),ECTEMP(20)
      DOUBLE PRECISION XMS,KTEMP,MAP_STEEL_ENERGY,MAP_STEEL_FTO1,CONST
      DOUBLE PRECISION XTO,H,S,H1,S1,F,AJ,AJ1,SLOPE,CORR,BXTO,CXTO,STORE
      INTEGER JTLOW,JTHIGH,J1,J2,JT
      INTEGER ITLOW,ITHIGH,IT,J,JJ,JJJ,II2,I,I1,II1,IZERO,JN
C
      IZERO = 0
C
C Initialize J1 and J2
C
      J1=0
      J2=0
C
C Guess a value of XTO, the carbon concentration (mole fraction) given by the
C   T-zero curve of the Temperature versus Carbon plot (phase diagram)
C
      XTO=0.07D+00
C
C H is the excess partial molar enthalpy of solution of carbon in austenite
C S is the excess partial molar entropy of solution of carbon in austenite
C H is in Joules per mole, S is in Joules per mole per K
C G. J. Shiflet, J. R. Bradley and H. I. Aaronson, Metallurgical Transactions
C   A, 9A (1978) 999.
C
      H=38575.0D+00
      S=13.48D+00
C
C This is to ensure that JTLOW, JTHIGH, JT are not changed on exit
C
      ITLOW=JTLOW
      ITHIGH=JTHIGH
      IT=JT
C
C H1 is the excess partial molar enthalpy of solution of carbon in ferrite
C S1 is the excess partial molar entropy of solution of carbon in ferrite
C H1 is in Joules per mole, S1 is in Joules per mole per K
C J. A. Lobo and  G. H. Geiger, Metallurgical Transactions A 7A (1976),
C  pages 1347 and 1359.
C
  100 DO 110 J= ITLOW,ITHIGH,IT
         J1=J1+1
         KTEMP=J
         IF (KTEMP .GE. 1000.0) THEN
            H1=105525.0D+00
            S1=45.34521D+00
         ELSE
            H1=111918.0D+00
            S1=51.44D+00
         ENDIF
C
C Calculate the free energy change for the transformation of austenite
C  to ferrite of the same chemical composition
C
         F=MAP_STEEL_ENERGY(KTEMP,T10,T20)
         AJ=1.0D+00-DEXP(-W/(R*KTEMP))
         AJ1=1.0D+00-DEXP(-W1/(R*KTEMP))
         FTO(J1)=MAP_STEEL_FTO1(H,S,X1,KTEMP,W,W1,H1,S1,F,AJ,AJ1,R)
C
C Calculate the temperature in degrees centigrade
C
         DCTEMP(J1)=KTEMP-273.0D+00
  110 CONTINUE
C
C Linear regression to obtain the free energy change for the transformation
C  of austenite to ferrite of the same chemical composition, as a
C  function of the temperature.
C
      IZERO = 0
      CALL MAP_UTIL_ANALY(J1,IZERO,CONST,SLOPE,CORR,DCTEMP,FTO)
C
C Calculate the bainite-start temperatures. Stored energy set at
C  400 J/mole -- see Bainite in Steels, H. K. D. H. Bhadeshia,
C  published by the Institute of Materials, London, (1992) pp. 1-458.
C
      BS=(-400.0-CONST)/SLOPE
C
C Calculate the martensite-start temperatures
C Modification made February 1991 to allow for large carbon concentrations
C Based on Metal Science paper on thermodynamics of martensitic transformations
C in plain carbon steels
C
C H. K. D. H. Bhadeshia, Metal Science 15 (1981) 175-177
C H. K. D. H. Bhadeshia, Metal Science 15 (1981) 178-180
C
      XMS=X1
      IF (XMS .GT. 0.0594D+00) XMS=0.0594D+00
      MS = (-1086.9D+00 + 2941.8D+00*XMS - 9.6016D+05*XMS*XMS
     &      + 2.5382D+07*XMS*XMS*XMS - 1.8488D+08*XMS*XMS*XMS*XMS
     &      - CONST)/SLOPE
C
C To avoid difficulties if transformation temperatures are too low
C     IF (ITLOW .LT. 140+273) ITLOW = 140+273
C
      ITHIGH=DINT(BS/20.0)*20+20+273
      IF ((ITHIGH-ITLOW)/IT .LT. 5) ITHIGH = 5*IT+ITLOW
      J2 = J2+1
      J1 = 0
      IF (J2 .EQ. 1) GOTO 100
C
C Start to calculate the T-zero and T-zero' curves of the
C  phase diagram (carbon [mole fraction] as a function of
C  temperature [K]).
C
C Begin with guessed values of BXT0 (T-zero carbon in mole fraction)
C  and CXTO (T-zero' carbon in mole fraction)
C
      I1=0
      BXTO = 0.07
      CXTO = BXTO
      DO 120 I = 313, 900, 20
         I1 = I1+1
         KTEMP = I
         IF (KTEMP .GE. 1000.0) THEN
            H1 = 105525.0D+00
            S1 = 45.34521D+00
         ELSE
            H1 = 111918.0D+00
            S1 = 51.44D+00
         ENDIF
         F = MAP_STEEL_ENERGY(KTEMP,T10,T20)
         AJ = 1.0D+00-DEXP(-W/(R*KTEMP))
         AJ1 = 1.0D+00-DEXP(-W1/(R*KTEMP))
         DCTEMP(I1) = KTEMP-273.0
         IZERO = 0
         JN = 9
         STORE = 0.0D0
         CALL MAP_STEEL_AXTO(H,S,H1,S1,BXTO,KTEMP,
     &                       W,W1,F,AJ,AJ1,STORE,JN,IZERO,R)
         IZERO = 0
         JN = 9
         STORE = 400D0
         CALL MAP_STEEL_AXTO(H,S,H1,S1,CXTO,KTEMP,
     &                       W,W1,F,AJ,AJ1,STORE,JN,IZERO,R)
         XXTO(I1) = BXTO
         YXTO(I1) = CXTO
         IF (CXTO .EQ. 0.0D+00) GOTO 130
  120 CONTINUE
C
130   JJJ = 0
      II1 = DINT(BS-80 +273)
      II2 = DINT(BS+60 +273)
      DO 140 JJ=1,100
         IF(DCTEMP(JJ)+273.0 .GT. II2 .OR. DCTEMP(JJ)+273.0 .LT.
     &   II1) THEN
            GOTO 140
         ELSE
            JJJ = JJJ+1
            VXTO(JJJ) = XXTO(JJ)
            WXTO(JJJ) = YXTO(JJ)
            ECTEMP(JJJ) = DCTEMP(JJ)
         ENDIF
  140 CONTINUE
C
C Fit straight lines to the T-zero and T-zero' curves
C XTOCN and XTOSL give the intercept and slope (respectively)
C   of the best fit line
C
      IZERO = 0
      CALL MAP_UTIL_ANALY(JJJ,IZERO,XTOCN,XTOSL,CORR,ECTEMP,VXTO)
      IZERO = 0
      CALL MAP_UTIL_ANALY(JJJ,IZERO,XTOCN2,XTOSL2,CORR,ECTEMP,WXTO)
C
      RETURN
      END
